@@ -20,9 +20,11 @@ c
 
       integer icon
 c
+
       icon = 1
 
 
+
       do while (icon .ne. 0)
 c          sample energy, mass, code (mometum is not given)
          call csampPrimary(incident.p, fin)
@@ -37,6 +39,10 @@ c            (in this case Za1ry == 'is' guaranteed).
 
          call csPrimAng(angle)
          call cmkInc(incident, angle)
+#if LABELING > 0
+         incident.info = 0
+         incident.label = 0
+#endif
 c           
          if(ObsPlane .eq. spherical) then
 c              reset position and angle.
@@ -53,9 +59,13 @@ c              reset position and angle.
                              ! which is not rejected. 
          endif
       enddo
+
+      
  10   continue
       end
-#ifdef NEXT486
+#if defined NEXT486
+#define IMAG_P dimag
+#elif defined PCLinux
 #define IMAG_P dimag
 #else
 #define IMAG_P imag
@@ -172,7 +182,7 @@ c         inc2.p = incidentp : for IBM, we must  write as follows.
       inc2.p.fm.p(4) = incidentp.fm.p(4)
       inc2.p.mass = incidentp.mass
       call cmkInc(inc2, angle)
-      call ciniTracking
+      call ciniTracking( inc2 )
       call cinitStack
       call cpush(inc2)
       end
@@ -194,7 +204,7 @@ c
 c                                       about primary
       
       call cmkInc2(incident)
-      call ciniTracking
+      call ciniTracking( incident ) 
       call cinitStack
       call cpush(incident)
       end
@@ -213,7 +223,7 @@ c
 #include  "Zincidentp.h"
 #include  "Zincidentv.h"
 #include  "Zearth.h"
-
+#include  "Zcode.h"
       record /track/ incident ! input/outut.
 c             must have E, m, code, subcode, charge
       record /coord/ angle    ! input. must have direction cos in the det. sys.
@@ -230,7 +240,9 @@ c           to see  cut off or to see if go out of Earth
          enddo
 c             charge conjugate
          incident.p.charge = -incident.p.charge
-         incident.p.subcode = -incident.p.subcode
+         if(incident.p.code .ne. kgnuc) then
+            incident.p.subcode = -incident.p.subcode
+         endif
       endif
 c           convert it to 'xyz' system
       call ctransVectZx(1, DetZaxis, DetXaxis, AngleAtObsCopy, 
@@ -243,9 +255,15 @@ c          HeightOfInj (=100 km); Normally OffsetHeight is 0.
 c         if the primary is to be directed to different height
 c         than the detector, make it non zero.
       zenithcos = - AngleAtObsCopy.r(3)
-      to100km = clenbetween2h(
-     *  ObsSites(NoOfSites).pos.radiallen + OffsetHeight, 
-     *  Eradius + HeightOfInj,  zenithcos )  ! we need zenith angle here
+      if(ObsPlane .ne. spherical) then
+         to100km = clenbetween2h(
+     *    ObsSites(NoOfSites).pos.radiallen + OffsetHeight, 
+     *    Eradius + HeightOfInj,  zenithcos )  ! we need zenith angle here
+      else
+c           dummy 
+         to100km = 10000.
+      endif 
+
 
 c         primary is going upward even if Reverse = 0.
       Upgoing = Reverse .eq. 0 .and. zenithcos .lt. 0. 
@@ -254,12 +272,14 @@ c         primary is going upward even if Reverse = 0.
       if(( Reverse .eq. 0 .and. zenithcos .lt. 0. 
      *  .and.  HeightOfInj .gt. ObsSites(NoOfSites).pos.height)
      *  .or. ( Reverse .ne. 0 .and. zenithcos .gt. 0.)) then
+         if(ObsPlane .ne. spherical) then
 c            distance to the conjugate point
-         to100km = to100km -
-     *    2*(ObsSites(NoOfSites).pos.radiallen + OffsetHeight)*
-     *    zenithcos
+            to100km = to100km -
+     *       2*(ObsSites(NoOfSites).pos.radiallen + OffsetHeight)*
+     *       zenithcos
 c             we should go reversed direction
-         to100km = - to100km
+            to100km = - to100km
+         endif
       endif
 c           injection point
       do i = 1, 3
@@ -276,12 +296,43 @@ c          set time 0
       incident.t = 0.
       incident.wgt = 1.       ! weight is 1.
 
-c           Next obs site
-      if( Upgoing ) then
-         incident.where = NoOfSites
-      else
-         incident.where = 1     
+
+      do i = 1, NoOfSites
+         if( HeightOfInj .gt. ObsSites(i).pos.height ) then
+            incident.where = i
+            goto 222
+         endif
+      enddo
+      if(HeightOfInj .lt. BorderHeightL) then
+         call cerrorMsg('Injection height is < BorderHeightL',0)
       endif
+      incident.where = NoOfSites + 1
+ 222  continue
+      if(HeightOfInj .gt. BorderHeightH) then
+         call cerrorMsg('Injection height is > BorderHeightH',0)
+      endif
+
+c           Next obs site
+c      if( Upgoing ) then
+ccc         incident.where = NoOfSites
+c         do i = NoOfSites, 1, -1
+c            if( HeightOfInj .lt. ObsSites(i).pos.height ) then
+c               incident.where = i
+c               goto 33
+c            endif
+c         enddo
+c         incident.where = 0
+c      else
+ccc         incident.where = 1     
+c         do i = 1, NoOfSites
+c            if( HeightOfInj .gt. ObsSites(i).pos.height ) then
+c               incident.where = i
+c               goto 33
+c            endif
+c         enddo
+c         incident.where = NoOfSites + 1
+c      endif
+c33   continue
 
       incident.asflag = 0
       incident.pos.colheight = Infty  ! latest nuc. collision height
@@ -409,6 +460,7 @@ c              icon !=0 ==> light cannot come with this angle
             ObsSites(i).minitime = 1.d10
          endif
       enddo
+
       from.t = 0.    ! reset time of incident track.
       end
 c     *************************** inquire incident particle
@@ -483,6 +535,7 @@ c
 c       ************************* see if geomagnetic cut or not.
         subroutine cifCutOff(icon)
         implicit none
+#include "Zglobalc.h"
 #include "Zobs.h"
 #include "Zobsp.h"
 #include "Ztrack.h"
@@ -500,26 +553,45 @@ c       ************************* see if geomagnetic cut or not.
          if(inc.p.charge .eq. 0) then
             icon = 0
          else
-c            angleatOb is down going ptcl's one, change sign
-            angleatOb.r(1) = - angleatOb.r(1)
-            angleatOb.r(2) = - angleatOb.r(2)
-            angleatOb.r(3) = - angleatOb.r(3)
-c             convert to theta fai in deg
-            call kdir2deg(angleatOb.r(1), angleatOb.r(2),
-     *      angleatOb.r(3), theta, azm)
+            if(Rdatafmt .le. 4) then
+c              angleatOb is down going ptcl's one, change sign
+               angleatOb.r(1) = - angleatOb.r(1)
+               angleatOb.r(2) = - angleatOb.r(2)
+               angleatOb.r(3) = - angleatOb.r(3)
+c                 convert to theta fai in deg
+               call kdir2deg(angleatOb.r(1), angleatOb.r(2),
+     *          angleatOb.r(3), theta, azm)
 c
-c               azm is given from the current x-axis  (+ is counter
-c              clock wise) The x-axis is XaxisFromSouth
-c            degrees from the south in counter clockwise.
-c            convert azm so that measured from the south
+c                 azm is given from the current x-axis  (+ is counter
+c                 clock wise) The x-axis is XaxisFromSouth
+c                 degrees from the south in counter clockwise.
+c                 convert azm so that measured from the south
 c
-            azm = mod(azm+ XaxisFromSouth, 360.d0)
-            if(ZenValue .eq. 'cos') then
-c                table zenith is in cos
-               zen = angleatOb.r(3)
+               azm = mod(azm+ XaxisFromSouth, 360.d0)
+               if(ZenValue .eq. 'cos') then
+c                    table zenith is in cos
+                  zen = angleatOb.r(3)
+               else
+                  zen = theta
+               endif
+            elseif(Rdatafmt .eq. 5) then
+c                 in this case, azm is longitude; zen is latitude or cos(lati)
+c       
+c               zen =  atan2( inc.pos.xyz.z, 
+c     *                sqrt(inc.pos.xyz.x**2+inc.pos.xyz.y**2) )
+               zen =  atan2( inc.pos.xyz.r(3),
+     *                sqrt(inc.pos.xyz.r(1)**2+inc.pos.xyz.r(2)**2) )
+               if(ZenValue .eq. 'cos') then
+                  zen = cos( zen-pi/2.0d0 )
+               else
+                  zen = zen*Todeg
+               endif
+c               azm = atan2(inc.pos.xyz.y, inc.pos.xyz.x)*Todeg
+               azm = atan2(inc.pos.xyz.r(2), inc.pos.xyz.r(1))*Todeg
             else
-               zen = theta
+               call cerrorMsg('dataformat for cut off invalid',0)
             endif
+
             p = sqrt(inc.p.fm.p(4)**2 - inc.p.mass**2)
             rig = p/abs(inc.p.charge)
             call crigCut(azm, zen, rig, prob)
@@ -531,3 +603,5 @@ c                table zenith is in cos
             endif
          endif
          end
+
+
